package util

import java.util.HashSet

object Characters {

    const val EOF = -1
    const val AT_SIGN = '@'
    const val BACKQUOTE = '`'
    const val BACKSLASH = '\\'
    const val COMMA = ','
    const val DASH = '-'
    const val DOUBLE_QUOTE = '\"'
    const val HASH = '#'
    const val LEFT_PARENTHESIS = '('
    const val LEFT_SQUARE_BRACKET = '['
    const val NEWLINE = '\n'
    const val PERIOD = '.'
    const val PLUS = '+'
    const val RIGHT_PARENTHESIS = ')'
    const val RIGHT_SQUARE_BRACKET = ']'
    const val SEMICOLON = ';'
    const val SINGLE_QUOTE = '\''
    const val UNICODE_ESCAPE = '\u001B'
    const val UNICODE_NULL = '\u0000'

    private val illegalIdentifierCharacters = HashSet<Char>()

    init {
        illegalIdentifierCharacters.add(AT_SIGN)
        illegalIdentifierCharacters.add(BACKQUOTE)
        illegalIdentifierCharacters.add(BACKSLASH)
        illegalIdentifierCharacters.add(COMMA)
        illegalIdentifierCharacters.add(DOUBLE_QUOTE)
        illegalIdentifierCharacters.add(HASH)
        illegalIdentifierCharacters.add(LEFT_PARENTHESIS)
        illegalIdentifierCharacters.add(LEFT_SQUARE_BRACKET)
        illegalIdentifierCharacters.add(PERIOD)
        illegalIdentifierCharacters.add(RIGHT_PARENTHESIS)
        illegalIdentifierCharacters.add(RIGHT_SQUARE_BRACKET)
        illegalIdentifierCharacters.add(SEMICOLON)
        illegalIdentifierCharacters.add(SINGLE_QUOTE)
    }

    fun isLegalIdentifierCharacter(c: Char) = !Character.isWhitespace(c) && !illegalIdentifierCharacters.contains(c)
    fun isLegalStringCharacter(c: Char) = true
    fun isNumberPrefix(c: Char) = c == DASH || c == PLUS
}
