package acceptance.fixture;

import application.LispMain;
import environment.RuntimeEnvironment;
import interpreter.LispInterpreter;
import interpreter.LispInterpreterBuilder;
import table.ExecutionContext;
import table.FunctionTable;
import util.Path;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.PrintStream;

public class LispInterpreterFixture {

    private static ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    private static ExecutionContext executionContext = ExecutionContext.INSTANCE;
    private static RuntimeEnvironment environment = RuntimeEnvironment.INSTANCE;
    private static LispInterpreter interpreter = null;

    public static void resetInterpreter() {
        cleanUp();
        buildInterpreter();
    }

    public static void cleanUp() {
        LispInterpreterBuilder.INSTANCE.reset();
        FunctionTable.INSTANCE.resetFunctionTable();
        executionContext.clearContext();
        environment.reset();
    }

    public static void buildInterpreter() {
        LispInterpreterBuilder.INSTANCE.setOutput(new PrintStream(outputStream));
        LispInterpreterBuilder.INSTANCE.setErrorOutput(new PrintStream(outputStream));
        LispInterpreterBuilder.INSTANCE.setNotInteractive();
        LispInterpreterBuilder.INSTANCE.setLanguageFileNames(LispMain.Companion.getLANGUAGE_FILE_NAMES());
        //        LispInterpreterBuilder.INSTANCE.setTerminationFunction(LispInterpreterFixture::terminate);
        //        LispInterpreterBuilder.INSTANCE.setErrorTerminationFunction(LispInterpreterFixture::terminateFromError);

        interpreter = LispInterpreterBuilder.INSTANCE.build();
    }

    public static void terminate() {}

    public static void terminateFromError() {
        throw new RuntimeException("Error Termination");
    }

    public String evaluateText(String input) {
        environment.setInputName("fitnesse");
        environment.setInput(new ByteArrayInputStream(input.getBytes()));
        environment.setPath("");

        return evaluate();
    }

    public String evaluateFile(String inputFile) throws FileNotFoundException {
        environment.setInputName(inputFile);
        environment.setInput(new FileInputStream(inputFile));
        environment.setPath(Path.INSTANCE.getPathPrefix(inputFile));

        return evaluate();
    }

    private String evaluate() {
        interpreter.interpret();
        String output = outputStream.toString();
        outputStream.reset();

        return output.trim();
    }
}
